/* @flow */

type OutputRequest = {
    address: string;
    amount: number; // in satoshis
};

type OutputResult = {
    type: ''
    address: string;
    amount: number;
} | {
    
};

// types copied from trezor.js
type OutputResult = {
    path: Array<number>;
    value: number;
    segwit: boolean;
} | {
    address: string;
    value: number;
};

type InputResult = {
    hash: Buffer;
    index: number;
    path: Array<number>;
    segwit: boolean;
    amount?: number; // only with segwit
};

type TxResult = {
    inputs: Array<InputResult>;
    outputs: Array<OutputResult>;
		permutation: OutputPermutation; // not in trezor.js, but needed for metadata saving
};

// Class for permutation of outputs
class OutputPermutation {
    // Permutation is an array,
    // where on Ith position is J, which means that Jth position in the original, unsorted
    // output array
    // is Ith in the new array.
    _permutation: Array<number>;

    constructor(outputs: Array<OutputResult>) {
        // I am "sorting range" - (0,1,2,3,...)
        // so I got the indexes and not the actual values inside

        // I am sorting by negative values, so I got the smaller last, bigger first
        const permutation = sortBy(range(outputs.length),
            i => [-(outputs[i].value)]);
        this._permutation = permutation;
    }

    shuffleOutputs(outputs: Array<OutputResult>): Array<OutputResult> {
        const shuffled = at(outputs, this._permutation);
        return shuffled;
    }

    forEach(f: (unsortedIx: number, sortedIx: number) => void) {
        this._permutation.forEach(f);
    }
}


function createTransaction(
    utxos: Array<UtxoInfo>,
    outputs: Array<OutputRequests>,
		changePath: Array<number>,
    fee: number, // in sat/byte, virtual size
    segwit: boolean
): TxResult {
    // TODO
}

function countMax(
    utxos: Array<UtxoInfo>,
    amounts: Set<number>, // in satoshis
    segwit: boolean
): number {
    // TODO
}
